<?php
/**
 * @file
 * @author rschaer
 *
 */
class FGApi {
  static private $instance = null;

  /**
   * We are implementing a singleton pattern
   */
  private function __construct() {
  }

  /**
   * 
   * @return FGApi
   */
  public function getInstance() {
    if (is_null(self :: $instance)) {
      self :: $instance = new self;
    }
    return self :: $instance;
  }
  
  /**
   * filtering the params and checking if they are complete
   * @param array $params
   * @return array
   *  - status -> boolean if need params are available
   *  - params
   */
  public function transformParams($params) {
    $checkedParams = array();
    foreach ($params as $key => $item) {
      $checkedParams[$key] = check_plain($item);
    }
    if (!isset($checkedParams['type'])) {
      $checkedParams['type'] = 'html';
    }
    if (!isset($checkedParams['path'])) {
      return array('status' => FALSE, 'params' => $checkedParams);
    }
    if (!isset($checkedParams['gallery'])) {
      return array('status' => FALSE, 'params' => $checkedParams);
    }
    
    return array('status' => TRUE, 'params' => $checkedParams);
  }
  
  /**
   * the path alias needs to be transformed so it can be
   * matched onto the internal path to find the pictures
   * @param array $params
   * @return string
   */
  public function createFgGalleryPath($params) {
    $settings = variable_get('fg_galleries', array());

    $ar_base_path = explode('/', $params['gallery']);

    //remove slashed that are too much
    if ($ar_base_path[count($ar_base_path)-1] == '') {
        unset($ar_base_path[count($ar_base_path)-1]);
      }
      if ($ar_base_path[0] == '') {
        unset($ar_base_path[0]);
      }
      $gallery = implode('/', $ar_base_path) . '/';


      $path = $settings[$gallery]['fg_path'] . $params['path'];

      $current_path_alias = $params['path'];
      $start = strlen($settings[$gallery]['fg_alias']);
      $path_new = substr($current_path_alias, ++$start);
      $path_new  = $settings[$gallery]['fg_path'] . $path_new;
      
      return $path_new;
  }
  /**
   * The main function for converting to an XML document.
   * Pass in a multi dimensional array and this recrusively loops through and builds up an XML document.
   *
   * @param array $data
   * @param string $rootNodeName - what you want the root node to be - defaultsto data.
   * @param SimpleXMLElement $xml - should only be used recursively
   * @return string XML
   */
  public function toXml ($data, $rootNodeName = 'data', $xml=null)  {
    if (empty($data)) {
      return false;
    }
    // turn off compatibility mode as simple xml throws a wobbly if you don't.
    if (ini_get('zend.ze1_compatibility_mode') == 1) {
      ini_set ('zend.ze1_compatibility_mode', 0);
    }

    if ($xml == null) {
      $xml = simplexml_load_string("<?xml version='1.0' encoding='utf-8'?><$rootNodeName />");
    }

    // loop through the data passed in.
    foreach ($data as $key => $value) {
      // no numeric keys in our xml please!
      if (is_numeric($key)) {
        // make string key...
        $key = "unknownNode_". (string) $key;
      }

      // replace anything not alpha numeric
      $key = preg_replace('/[^a-z]/i', '', $key);

      // if there is another array found recrusively call this function
      if (is_array($value)) {
        $node = $xml->addChild($key);
        // recrusive call.
        FGApi::toXml($value, $rootNodeName, $node);
      }
      else {
        // add single node.
        $value = htmlentities($value);
        $xml->addChild($key, $value);
      }

    }
    // pass back as string. or simple xml object if you want!
    return $xml->asXML();
  }
  
  /**
   * Transform a given array into a cooliris XML structure
   * Sampe XML looks like this
   *  <?xml version="1.0" encoding="utf-8" standalone="yes"?>
      <rss version="2.0" xmlns:media="http://search.yahoo.com/mrss/"
      xmlns:atom="http://www.w3.org/2005/Atom">
      <channel>
          <item>
               <title>Picture A</title>
               <media:description> This one's my favorite.</media:description>
               <link>pl_images/A.jpg</link>
               <media:thumbnail url="http://example.com/pl_thumbs/A.jpg"/>
               <media:content url="http://example.com/pl_images/A.jpg"/>
          </item>
          <item>
              <title>Video B</title>
              <link>http://example.com/pl_images/B.jpg</link>
              <media:thumbnail url="http://example.com/pl_thumbs/B.jpg"/>
              <media:content type="video/x-flv"
              url="http://example.com/pl_images/B.flv"/>
          </item>
      </channel>
      </rss>
   * @param string $images
   */
  public function toCoolirixXml($images) {
    global $base_url;
    /*$xml = simplexml_load_string('<?xml version="1.0" encoding="utf-8" standalone="yes"?><rss version="2.0" xmlns:media="http://search.yahoo.com/mrss/"
    //  xmlns:atom="http://www.w3.org/2005/Atom" />');*/
    $xml = '<?xml version="1.0" encoding="utf-8" standalone="yes"?><rss version="2.0" xmlns:media="http://search.yahoo.com/mrss/"
      xmlns:atom="http://www.w3.org/2005/Atom" />';
    $xml .= '<channel>';
    /*print '<pre>';
    print_r($images);
    exit();*/
    $channel = '';
    foreach ($images as $image) {
      //$itemXml = new SimpleXMLElement('<item/>');
      $channel .= '<item>';
      //$itemXml->addChild('title', $image->getName());
      $channel .= '<title>' . $image->getName() . '</title>';
      //$itemXml->addChild('link', $image->getPath());
      $channel .= '<link>' . $image->getPath() . '</link>';
      if (variable_get('fast_gallery_ic_thumb', '') != '') {
        $image_path = imagecache_create_url(variable_get('fast_gallery_ic_thumb', ''),  $image->getPath());
      }
      else {
        drupal_set_message(t('Imagecache is needed! Please configure an imagecache preset'), 'error');
      }
      if (variable_get('fast_gallery_ic_full', '') != '') {
        $image_full = imagecache_create_url(variable_get('fast_gallery_ic_full', ''),  $image->getPath());
      }
      else {
        drupal_set_message(t('Imagecache is needed! Please configure an imagecache preset'), 'error');
      }
      //$itemXml->addChild('media:thumbnail', '')->addAttribute('url', $image_path);
      $channel .= '<media:thumbnail url="' . $image_path . '" />'; 
      //$itemXml->addChild('media:content', '')->addAttribute('url', $image_path);
      $channel .= '<media:content url="' . $image_full . '" />';
      $channel .= '</item>';
    }
    $xml .= $channel . '</channel></rss>';
    
    //$xml->addChild('channel', $channel);
    //return $xml->asXML();
    return $xml;
  }
}
